@@ -2,7 +2,7 @@ package Catmandu::Store::ElasticSearch::Bag;
 
 use Catmandu::Sane;
 
-our $VERSION = '1.01';
+our $VERSION = '1.02';
 
 use Catmandu::Hits;
 use Cpanel::JSON::XS qw(encode_json decode_json);
@@ -224,20 +224,36 @@ sub search {
 
     my $id_key = $self->id_key;
 
-    my $start = delete $args{start};
-    my $limit = delete $args{limit};
-    my $bag   = delete $args{reify};
+    my $start  = delete $args{start};
+    my $limit  = delete $args{limit};
+    my $scroll = delete $args{scroll};
+    my $bag    = delete $args{reify};
 
     if ($bag) {
         $args{fields} = [];
     }
 
-    my $res = $self->store->es->search(
+    my %es_args = (
         index => $self->index,
         type  => $self->type,
-        body  => {%args, from => $start, size => $limit,},
+        body  => {%args, size => $limit,},
     );
 
+    if (defined $scroll) {
+        $es_args{scroll} = $scroll;
+    }
+    else {
+        $es_args{body}{from} = $start;
+    }
+
+    my $res;
+    if ($args{scroll_id}) {
+        $res = $self->store->es->scroll(%es_args);
+    }
+    else {
+        $res = $self->store->es->search(%es_args);
+    }
+
     my $docs = $res->{hits}{hits};
 
     my $hits
@@ -263,6 +279,7 @@ sub search {
 
     $hits = Catmandu::Hits->new($hits);
 
+    $hits->{scroll_id} = $res->{_scroll_id} if exists $res->{_scroll_id};
     for my $key (qw(facets suggest aggregations)) {
         $hits->{$key} = $res->{$key} if exists $res->{$key};
     }
